<?php

namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Teknisa\MAN\Services;
use Teknisa\MAN\Utilities;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria; 

class ContaClienteConsFamilia extends CrudWithParameterBag {
    
    protected $dataSourceName = 'vnd09500_manutencaocontaclienteconsfamilia';
    protected $parameterBagColumns = array("CDCONSUMIDOR","CDCLIENTE","NRORG","TCDFILIAL","CDFILIAL");
    protected $util;
    
    public function __construct (Manager $dataSourceManager,ParameterBag $parameterBag, Utilities\Util $util){
        $this->util = $util;
        parent::__construct($dataSourceManager,$parameterBag);
    }
    
    public function find (DTO\Request\Filter $request, DTO\Response $response){
        $conditions = $this->util->getParamsAsArray($request);
        $dataSourceFilter = array();
        $likeableConditions = $this->util->getLikeConditions($request);
        unset($conditions['NMRAZSOCCLIE']);

        
        foreach ($likeableConditions as $condition) {
			if(array_key_exists($condition['columnName'], $conditions)) {
				 unset($conditions[$condition['columnName']]);
			}
		}
        
        $conditions = $this->util->buildParams('CDFILIAL',$conditions);
        $newRequest = $this->util->createNewZeedhiFilterRequestObject($request, $this->dataSourceName, $conditions);
        foreach ($likeableConditions as $condition) {
            if($condition['columnName'] != "NMRAZSOCCLIE"){
                $newRequest->getFilterCriteria()->addCondition($condition['columnName'], $condition['operator'], $condition['value']);    
            }
		}
        parent::find($newRequest, $response);
    }
    
    public function findClientePorUnidade(DTO\Request\Filter $request, DTO\Response $response) {
        $result = [];
        $dataSourceName = $this->dataSourceName = 'cdClientePorUnidade';
        
        $conditions = $this->util->getParamsAsArray($request);
        
        if(isset($conditions['CDFILIAL'])) {
            $conditions['CDFILIAL'] = implode('_', $conditions['CDFILIAL']);
            
            foreach($conditions as $key => $value) {
                $dataSourceFilter[] = $condition = array(
                    "columnName" => $key,
                    "operator"   => "=",
                    "value"      => $value
                );
            }
            
            // para o CTRL+F funcionar
            for($i = 0; $i < count($conditions); $i++) {
                if(strpos($dataSourceFilter[$i]["columnName"], '|') != false) {
                    $dataSourceFilter[$i]["operator"] = "MAPPED_LIKE_ALL";
                }
            }
            
            $newRequest = new DTO\Request\Filter (
                new FilterCriteria(
                    $dataSourceName,
                    $dataSourceFilter,
                    $request->getFilterCriteria()->getPage(),
                    $request->getFilterCriteria()->getPageSize()
                ),
                $request->getMethod(),
                $request->getRoutePath(),
                $request->getUserId()
            );
            
            parent::find($newRequest, $response);
        }
        
        return $result;
    }
}