<?php
namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Response\Message;
use Teknisa\MAN\Utilities\Util;
use Teknisa\MAN\Services;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\Util\Functions;

class Layout extends CrudWithParameterBag {
    
    protected $util,
              $parameterBag,
              $layoutService,
              $fileUploaderService;
    
    protected $dataSourceNameArray = array (
        'filterLayout'               => 'vnd80600_configterminalcaixa',
        'filterCaixa'                => 'vnd80600_configterminalcaixa_caixasAssoc',
        'filterGrupo'                => 'vnd80600_configterminalcaixa_layoutGrupo',
        'filterProdutos'             => 'vnd80600_configterminalcaixa_produtos_registraveis',
        'filterGrupoProduto'         => 'vnd80600_configterminalcaixa_layoutProduto',
        'filterGrupoSub'             => 'vnd80600_configterminalcaixa_layoutSubGrupo',
        'filterGrupoSubProduto'      => 'vnd80600_configterminalcaixa_layoutProdutoSub',
        'filterGrupoRec'             => 'vnd80600_configterminalcaixa_layoutGrupoRec',
        'filterLayoutRec'            => 'vnd80600_configterminalcaixa_layoutRec',
        'filterCodigoLayout'         => 'vnd80600_configterminalcaixa_codigoLayout',
        'filterProdutos_getPosicoes' => 'vnd80600_configterminalcaixa_layoutGrupo_getPosicoes',
        'findFamilia'                => 'vnd80600_configterminalcaixa_layoutGrupo_getFamilia',
        'findModulos'                => 'vnd80600_configterminalcaixa_layoutGrupo_getModulos',
        'findProdRanged'             => 'vnd80600_configterminalcaixa_produtosRanged',
        'saveProdCadAut'             => 'vnd80600_configterminalcaixa_layoutGrupo_cadAut',
        'filterGrupoProdutoCadAut'   => 'vnd80600_configterminalcaixa_layoutGrupoProduto_cadAut',
        'filterProdutosCadAud'       => 'vnd80600_configterminalcaixa_produtos_registraveis_cadAut',
        'findVendaItemProdutos'      => 'confTelaSugestaoVendaItemProdutos'
    );
    
    protected $parameterBagColumnsArray = array (
        'filterLayout'               =>  array('NRORG', 'CDFILIAL', 'CDLOJA'),
        'filterCaixa'                =>  array('NRORG', 'CDFILIAL', 'NRCONFTELA', 'DTINIVIGENCIA'),
        'filterLoja_Tela'            =>  array('NRORG', 'CDFILIAL', 'CDLOJA', 'NRCONFTELA', 'DTINIVIGENCIA'),
        'filterProdutosSugestaoItem' =>  array('NRORG', 'CDFILIAL', 'CDLOJA', 'NRCONFTELA'),
        'filterProdutos'             =>  array('NRORG', 'CDFILIAL'),
        'filterGrupo'                =>  array('NRORG', 'CDFILIAL', 'NRCONFTELA', 'NRPGCONFTAUX', 'NRBUTTONAUX', 'CDLOJA', 'DTINIVIGENCIA'),
        'filterGrupoProd'            =>  array('NRORG', 'CDFILIAL', 'NRCONFTELA', 'NRPGCONFTAUX', 'NRBUTTONAUX', 'IDTPBUTONAUX', 'CDLOJA', 'DTINIVIGENCIA'),
        'filterGrupoSubProduto'      =>  array('NRORG', 'CDFILIAL', 'NRCONFTELA', 'NRPGCONFTAUX', 'NRBUTTONAUX', 'NRPGCONFTAUX2', 'NRBUTTONAUX2', 'DTINIVIGENCIA'),
        'filterGrupoRec'             =>  array('NRORG', 'CDFILIAL', 'NRCONFTELA', 'DTINIVIGENCIA'),
        'filterFamilia'              =>  array('NRORG', 'CDFILIAL'),
        'filterModulos'              =>  array('NRORG', 'CDFILIAL'),
        'filterGrupoProdCadAut'      =>  array('NRORG', 'CDFILIAL', 'CDLOJA', 'NRCONFTELA', 'NRPGCONFTELA', 'DTINIVIGENCIA'),
        'filterProdutosCadAud'       =>  array('NRORG', 'CDFILIAL'),
        'filterProdutosCadAutomatico'=>  array('NRORG', 'CDNVPRODUTO'),
    );

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, Util $util, Services\Layout $layoutService, Services\FileUploaderService $fileUploaderService, \Doctrine\ORM\EntityManager $entityManager) {
        parent::__construct($dataSourceManager, $parameterBag);
        $this->util = $util;
        $this->layoutService = $layoutService;
        $this->fileUploaderService = $fileUploaderService;
        $this->entityManager = $entityManager;
    }
    
    public function saveDefaulLayout ($row, $response, $dataSourceNameSave) {
        try {
            $dataSet = new DataSet($dataSourceNameSave, array($row));
            $persistedRows = $this->dataSourceManager->persist($dataSet);
            $response->addDataSet(new DataSet($dataSourceNameSave, $persistedRows));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }    
    }
    
    public function deleteDefaultLayout ($request, $response, $dataSourceNameDelete) {
        if(method_exists($request,"getDataSet")) {
            parent::delete($request, $response);
        }
        try {
            $row = $request->getRow();
            $dataSet = new DataSet($dataSourceNameDelete, array($row));
            $deletedRows = $this->dataSourceManager->delete($dataSet);
            $response->addDataSet($dataSet);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function findLayout (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterLayout'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['filterLayout'];
        parent::find($request, $response);
    }
    
    public function saveLayout (DTO\Request\DataSet $request, DTO\Response $response) {
        self::saveDefaulLayout($request->getDataSet()->getRows()[0], $response, $this->dataSourceNameArray['filterLayout']);
    }
    
    public function findCaixa (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterCaixa'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['filterCaixa'];
        parent::find($request, $response);
    }
    
    public function saveCaixa (DTO\Request\DataSet $request, DTO\Response $response) {
        $row = $request->getDataSet()->getRows()[0];
        
        for($i = 0; $i < count($row['CDCAIXA']); $i++) {
            //verifica caixa associado
            if($this->layoutService->verificaCaixa($row['CDFILIAL'], $row['CDFILICONFTE'], $row['DTINIVIGENCIA'], $row['CDCAIXA'][$i], $row['NRORG'])){
                $this->layoutService->updateCaixa($row['NRCONFTELA'], $row['CDFILICONFTE'], $row['CDFILIAL'], $row['CDCAIXA'][$i], $row['NRORG'], $row['DTINIVIGENCIA']);     
            }else{
               $mensagem =  'Caixa '. $row['CDCAIXA'][$i]. ' já associado em outra Configuração com mesma data de vigência'; 
               $response->addNotification(new Notification($mensagem, Notification::TYPE_ERROR));       
            }
        }
    }
    
    public function deleteCaixa (DTO\Request\Row $request, DTO\Response $response) {
        self::deleteDefaultLayout($request, $response, $this->dataSourceNameArray['filterCaixa']);
    }  
    
    public function deleteCaixas (DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterCaixa'];
        parent::delete($request, $response);
    }  
    
    public function findGrupo (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterGrupo'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['filterLoja_Tela'];
        parent::find($request, $response);
    }
    
   public function resizeImage(DTO\Request\Row $request, DTO\Response $response){
        $row = $this->util->getParamsAsArray($request);
        list($width_orig, $height_orig, $type) = getimagesize($row['IMAGE'][0]['b64File']);
            if(  $width_orig == $height_orig &&  ($width_orig >= 400 && $width_orig <= 800) ){
                list($typePNG, $data) = explode(';', $row['IMAGE'][0]['b64File']);
                list(, $data)      = explode(',', $data);
                $curl_parameters = array('imagem' => ($data));
                $curl_parameters = json_encode($curl_parameters);
                if($width_orig != 400){            
                    $cr = curl_init();
                    curl_setopt($cr, CURLOPT_URL, 'https://rm.teknisa.com/tekimage/');
                    curl_setopt($cr, CURLOPT_CUSTOMREQUEST, 'POST');
                    curl_setopt($cr, CURLOPT_HTTPPROXYTUNNEL, 0);
                    curl_setopt($cr, CURLOPT_POSTFIELDS, $curl_parameters);
                    curl_setopt($cr, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);
                    curl_setopt($cr, CURLOPT_RETURNTRANSFER, true);
                    curl_setopt($cr, CURLOPT_TIMEOUT, 30);
                    $return = json_decode(curl_exec($cr));
                    if(!!$return){
                        $row['IMAGE'][0]['b64File'] = 'data:image/png;base64,'.($return->b64File);
                    } else {
                        $response->addMessage(new Message("Não foi possivel otimizar a imagem. Servidor indisponível.", "W"));
                    }
                }
            } else {
                $response->addMessage(new Message("Imagem Inválida. A imagem deve possuir lados iguais que devem estar entre 400px a 800px.", "E"));
            }
        $response->addDataset(new DataSet('newRow', array($row)));
    }
    
    public function saveGrupo(DTO\Request $request, DTO\Response $response) {
        $rows = $request->getDataSet()->getRows();
        
        if(count($rows) === 1 ) {
            $row = $rows[0];
                $this->fileUploaderService->setImageForSave($row, 'DSENDEIMG', 'IMAGE');
                self::saveDefaulLayout($row, $response, $this->dataSourceNameArray['filterGrupo']);
        } else {
            throw new \Exception('Só é possível salvar um registro de cada vez.');
        }
    }
    
    public function deleteGrupo(DTO\Request\Row $request, DTO\Response $response) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try{
            $row = $request->getRow();
            $params = array(
                ':NRORG'        => $row['NRORG'],
                ':CDFILIAL'     => $row['CDFILIAL'],
                ':CDLOJA'       => $row['CDLOJA'],
                ':NRCONFTELA'   => $row['NRCONFTELA'],
                ':NRPGCONFTAUX' => $row['NRPGCONFTELA'],
                ':NRBUTTONAUX'  => $row['NRBUTTON']
            );
            
            $produtosDoGrupo = $this->layoutService->verificaGrupoVazio($params);
            $subGrupoDoGrupo = $this->layoutService->verificaGrupoVazioSubGrupo($params);
           
            if(empty($produtosDoGrupo) && empty($subGrupoDoGrupo)) {
                self::deleteDefaultLayout($request, $response, $this->dataSourceNameArray['filterGrupo']);
            } else {
                $validProduto = true;
                $validSubGrupo = true;
                
                foreach($produtosDoGrupo as $key => $value){
                    if(!empty($produtosDoGrupo[$key]['CDPRODUTO'])){
                        $validProduto = false;
                    }
                }
    
                if(count($subGrupoDoGrupo) > 0){
                    $param = array(
                        ':NRORG'        => $row['NRORG'],
                        ':CDFILIAL'     => $row['CDFILIAL'],
                        ':NRCONFTELA'   => $row['NRCONFTELA'],
                        ':DTINIVIGENCIA'=> $row['DTINIVIGENCIA'],
                        ':NRBUTTONAUX2' => $row['NRBUTTON']
                    );
                    $this->layoutService->DeleteProdSub($param);
                    $validSubGrupo = true;
                }
                
                $param = array(
                        ':NRORG'        => $row['NRORG'],
                        ':CDFILIAL'     => $row['CDFILIAL'],
                        ':NRCONFTELA'   => $row['NRCONFTELA'],
                        ':NRBUTTONAUX'  => $row['NRBUTTON'],
                        ':NRPGCONFTAUX' => $row['NRPGCONFTELA'],
                        ':DTINIVIGENCIA'=> $row['DTINIVIGENCIA']
                    );
                $subgrupo =  $this->layoutService->VerificaSubGrupo($param);
    
                if(!$validProduto || !empty($subgrupo)){
                    $params = array(
                        ':NRORG'        => $row['NRORG'],
                        ':CDFILIAL'     => $row['CDFILIAL'],
                        ':NRCONFTELA'   => $row['NRCONFTELA'],
                        ':NRBUTTONAUX'  => $row['NRBUTTON'],
                        ':NRPGCONFTAUX' => $row['NRPGCONFTELA'],
                        ':DTINIVIGENCIA'=> $row['DTINIVIGENCIA']
                    );
            	    $this->layoutService->deletecascade($params);
                	$validProduto = true;
                }
                
                if($validProduto && $validSubGrupo){
                    self::deleteDefaultLayout($request, $response, $this->dataSourceNameArray['filterGrupo']);
                }else{
                    $response->setCriticalError(new DTO\Response\Error('Exclusão não permitida! 
                        O registro possui registros filhos.', '00001'));                
                }
            }
            $connection->commit();
        }
        catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
    
    public function findProdutos (DTO\Request\Filter $request, DTO\Response $response) {
        $conditions = $this->util->getParamsAsArray($request);
        $cdArvProd = isset($conditions['CDARVPROD']) ? $conditions['CDARVPROD'] : false;
        unset($conditions['CDARVPROD']);
        $parametros = $this->layoutService->getProdutoFamilia($conditions);
        if(!is_null($parametros)){
            $dataSourceFilter = array();
            $likeableConditions = $this->util->getLikeConditions($request);
            foreach ($likeableConditions as $condition) {
    			if(Functions::arrayKeyExists($condition['columnName'], $conditions)) {
    				 unset($conditions[$condition['columnName']]);
    			}
    		}
            $conditions = $this->util->buildParamsNotArray('CDFAMILISALD',$conditions);
            $conditions = $this->util->buildParamsNotArray('CDMODULOPROD',$conditions);
            $conditions['CDSEGMER'] = $parametros['CDSEGMER'];
            $this->dataSourceName = 'vnd80600_configterminalcaixa_produtos_registraveis';
            $newRequest = $this->util->createNewZeedhiFilterRequestObject($request, $this->dataSourceName, $conditions);
            $this->parameterBagColumns = $this->parameterBagColumnsArray['filterProdutosCadAutomatico'];
            if($cdArvProd){
                $newRequest->getFilterCriteria()->addCondition('CDARVPROD', '>=', $cdArvProd);
            }
            foreach ($likeableConditions as $condition) {
    			$newRequest->getFilterCriteria()->addCondition($condition['columnName'], $condition['operator'], $condition['value']);
    		}
            
            parent::find($newRequest, $response);
        }
        
    }
    
    public function findGrupoProduto (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterGrupoProduto'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['filterGrupoProd'];
        parent::find($request, $response);
    }
    
    public function saveGrupoProduto(DTO\Request\DataSet $request, DTO\Response $response) {
        $rows = $request->getDataSet()->getRows();
        
        if(count($rows) > 0) {
            $row = $rows[0];
            $this->fileUploaderService->setImageForSave($row, 'DSENDEIMG', 'IMAGE');
            
            $this->dataSourceName = $this->dataSourceNameArray['filterGrupoProduto'];
            
            $paginaAntiga = substr($row['CDGRUPO'], 1);
            
            $params = array(
                ':CDFILIAL'      => $row['CDFILIAL'],
                ':NRCONFTELA'    => $row['NRCONFTELA'],
                ':DTINIVIGENCIA' => $row['DTINIVIGENCIA'],
                ':IDTPBUTTON'    => $row['IDTPBUTTON'],
                ':NRPGCONFTELA'  => $row['NRPGCONFTELA'],
                ':NRPGCONFTAUX'  => $row['NRPGCONFTAUX'],
                ':NRBUTTONAUX'   => $row['NRBUTTONAUX'],
                ':IDTPBUTONAUX'  => $row['IDTPBUTONAUX'],
                ':NRBUTTON'      => $row['NRBUTTON']
            );
            
            // verifica se ja existe a posicao ocupada
            $podeSalvar = $this->layoutService->verificaPaginaEPosicaoProdutoSubGrupo($params, $paginaAntiga,$this->layoutService::VERIFICA_PAGINA_E_POSICAO_PRODUTO);
            
            if($podeSalvar) {
                if((Functions::arrayKeyExists('ANTG_NRBUTTON', $row)) && 
                  ($row['ANTG_NRBUTTON'] !== '') && 
                  ($row['NRBUTTON']      !== $row['ANTG_NRBUTTON'])) {
                    $aux = $row['NRBUTTON'];     
                    $row['NRBUTTON'] = $row['ANTG_NRBUTTON'];
                    self::delete($request, $response);
                    $row['NRBUTTON'] = $aux;
                    $row['__is_new'] = true;
                }
                
                self::saveDefaulLayout($row, $response, $this->dataSourceNameArray['filterGrupoProduto']);
            } else {
                throw new \Exception('Não é possível salvar um produto na mesma posição.');
            }
        }
    }
    
    public function deleteGrupoProduto(DTO\Request\Row $request, DTO\Response $response) {
        
        
        self::deleteDefaultLayout($request, $response, $this->dataSourceNameArray['filterGrupoProduto']);
    }
    
    public function findGrupoSub (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterGrupoSub'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['filterGrupo'];
        parent::find($request, $response);
    }
    
    public function saveGrupoSub(DTO\Request\DataSet $request, DTO\Response $response) {        
        $rows = $request->getDataSet()->getRows();
        
        if(count($rows) === 1 ) {
            $row = $rows[0];
            if(isset($row['IMAGE'][0]['path'])){
                $row['DSENDEIMG'] = $row['IMAGE'][0]['path'];
            }
            $this->fileUploaderService->setImageForSave($row, 'DSENDEIMG', 'IMAGE');
            
            self::saveDefaulLayout($row, $response, $this->dataSourceNameArray['filterGrupoSub']);
        } else {
            throw new \Exception('Só é possível salvar um registro de cada vez.');
        }
    }
    
    public function deleteGrupoSub(DTO\Request\Row $request, DTO\Response $response) {
        
        $row = $request->getRow();
        $params = array(
            ':NRORG'        => $row['NRORG'],
            ':CDFILIAL'     => $row['CDFILIAL'],
            ':CDLOJA'       => $row['CDLOJA'],
            ':NRCONFTELA'   => $row['NRCONFTELA'],
            ':NRPGCONFTAUX' => $row['NRPGCONFTELA'],
            ':NRBUTTONAUX'  => $row['NRBUTTON'],
            ':NRPGCONFTAUX2' => $row['NRPGCONFTAUX'],
            ':NRBUTTONAUX2'  => $row['NRBUTTONAUX']
        );
        
        $produtoDoSubGrupo = $this->layoutService->verificaSubGrupoVazioProduto($params);
        $validSubGrupo = true;
        if(!empty($produtoDoSubGrupo)){
            $params = array(
            ':NRORG'        => $row['NRORG'],
            ':CDFILIAL'     => $row['CDFILIAL'],
            ':NRCONFTELA'   => $row['NRCONFTELA'],
            ':NRPGCONFTAUX' => $row['NRPGCONFTELA'],
            ':NRBUTTONAUX'  => $row['NRBUTTON'],
            ':NRPGCONFTAUX2' => $row['NRPGCONFTAUX'],
            ':NRBUTTONAUX2'  => $row['NRBUTTONAUX'],
            ':DTINIVIGENCIA' =>$row['DTINIVIGENCIA']
           

        );

         $this->layoutService->deletecascadesub($params);
         $validSubGrupo=true;
        }
        
        if($validSubGrupo){
            self::deleteDefaultLayout($request, $response, $this->dataSourceNameArray['filterGrupoSub']);
        } else {
            $response->setCriticalError(new DTO\Response\Error('Exclusão não permitida! O registro possui registros filhos.', '00001'));
        }
    }
    
    public function findGrupoSubProduto (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterGrupoSubProduto'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['filterGrupoSubProduto'];
        parent::find($request, $response);
    }
    
    public function saveGrupoSubProduto(DTO\Request\DataSet $request, DTO\Response $response) {
        $rows = $request->getDataSet()->getRows();
        
        if(count($rows) === 1 ) {
            $row = $rows[0];
            
            if(isset($row['IMAGE'][0]['path'])){
                $row['DSENDEIMG'] = $row['IMAGE'][0]['path'];
            }
            $this->fileUploaderService->setImageForSave($row, 'DSENDEIMG', 'IMAGE');
            
            $paginaAntiga = substr($row['CDGRUPO'], 1);
            
            // pega os parametros
            $params = array(
                ':CDFILIAL'      => $row['CDFILIAL'],
                ':NRCONFTELA'    => $row['NRCONFTELA'],
                ':DTINIVIGENCIA' => $row['DTINIVIGENCIA'],
                ':IDTPBUTTON'    => $row['IDTPBUTTON'],
                ':NRPGCONFTELA'  => $row['NRPGCONFTELA'],
                ':NRPGCONFTAUX'  => $row['NRPGCONFTAUX'],
                ':NRBUTTONAUX'   => $row['NRBUTTONAUX'],
                ':IDTPBUTONAUX'  => $row['IDTPBUTONAUX'],
                ':NRBUTTON'      => $row['NRBUTTON'],
                ':NRORG'         => $row['NRORG'],
                ':IDTPBUTONAUX2' => $row['IDTPBUTONAUX2'],
                ':NRBUTTONAUX2'  => $row['NRBUTTONAUX2'],
                ':NRPGCONFTAUX2' => $row['NRPGCONFTAUX2']
  
            );
            
            // verifica se ja existe a posicao ocupada
            $podeSalvar = $this->layoutService->verificaPaginaEPosicaoProdutoSubGrupo($params, $paginaAntiga,$this->layoutService::VERIFICA_PAGINA_E_POSICAO_PRODUTO_SUBGRUPO);
            
            if($podeSalvar) {
                $requestNovo = $request;
                $requestNovo->getDataSet()->getRows()[0]['NRBUTTON'] = $paginaAntiga;
                
                // excluindo produto
                $this->dataSourceName = $this->dataSourceNameArray['filterGrupoSubProduto'];
                self::delete($requestNovo, $response);
                
                $row['NRBUTTON'] = $params[':NRBUTTON'];
                
                // salvando o novo produto
                $this->layoutService->insertProdutoSubGrupo($row);
            } else {
                throw new \Exception('Não é possível salvar um produto na mesma posição.');
            }
        } else {
            throw new \Exception('Só é possível salvar um registro de cada vez.');
        }
    }
    
    public function deleteGrupoSubProduto(DTO\Request\Row $request, DTO\Response $response) {
        self::deleteDefaultLayout($request, $response, $this->dataSourceNameArray['filterGrupoSubProduto']);
    }
    
    public function findGrupoRec (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterGrupoRec'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['filterGrupoRec'];
        parent::find($request, $response);
    }
    
    public function saveGrupoRec(DTO\Request\DataSet $request, DTO\Response $response) {
        self::saveDefaulLayout($request->getDataSet()->getRows()[0], $response, $this->dataSourceNameArray['filterGrupoRec']);
    }
    
    public function deleteGrupoRec (DTO\Request\Row $request, DTO\Response $response) {
        //delete tipo de recebimentos do grupo de recebimento, antes de deletar grupo
        
        $row = $request->getRow();
        $params = array(
            ':NRORG'        => $row['NRORG'],
            ':CDFILIAL'     => $row['CDFILIAL'],
            ':NRCONFTELA'   => $row['NRCONFTELA'],
            ':NRPGCONFTAUX' => $row['NRPGCONFTELA'],
            ':NRBUTTONAUX'  => $row['NRBUTTON'],
            ':DTINIVIGENCIA' => $row['DTINIVIGENCIA']
        );
        
        $validSubGrupo = $this->layoutService->deletecascadetpReceb($params);
        if($validSubGrupo){
             self::deleteDefaultLayout($request, $response, $this->dataSourceNameArray['filterGrupoRec']);
        } else {
            $response->setCriticalError(new DTO\Response\Error('Exclusão não permitida! O registro possui registros filhos.', '00001'));
        }
    }
    
    public function findLayoutRec (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterLayoutRec'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['filterGrupo'];
        parent::find($request, $response);
    }
    
    public function saveLayoutRec(DTO\Request\DataSet $request, DTO\Response $response) {
        $rows = $request->getDataSet()->getRows();
        
        if(count($rows) === 1 ) {
            $row = $rows[0];
            $this->fileUploaderService->setImageForSave($row, 'DSIMAGEMBUT', 'IMAGE');
            
            self::saveDefaulLayout($request->getDataSet()->getRows()[0], $response, $this->dataSourceNameArray['filterLayoutRec']);
        } else {
            throw new \Exception('Só é possível salvar um registro de cada vez.');
        }
    }
    
    public function deleteLayoutRec (DTO\Request\Row $request, DTO\Response $response) {
        self::deleteDefaultLayout($request, $response, $this->dataSourceNameArray['filterLayoutRec']);
    }

    public function findNewNrconftela (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterCodigoLayout'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['filterProdutos'];
        parent::find($request, $response);
    }
    
    /*
        Método criado para encontrar quais posições estão disponíveis dentro de uma página pré determinada.
        Ele recebe como parâmetro alguns dados trazidos de LayoutController.js.
        Utiliza a query BUSCA_POSICOES_DISPONIVEIS para encontrar as posições que estão ocupadas
        Então, compara com as posições que ele pode inserir, por padrão.
        Se a posição que trouxe de Layout (Service) for igual a alguma posição doo array, 
        ele remove essa opção do array utilizando uma função do php chamada array_slice. 
    */
    public function findPosicaoDisponivel (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterProdutos_getPosicoes'];
        $retorno = [];
        $conditions = $request->getFilterCriteria()->getConditions();
        if(count($conditions) > 0) {
            $params = array(
    			':NRORG'  		  => $conditions[0]['value'],
    			':CDFILIAL'       => $conditions[1]['value'],
    			':NRCONFTELA'     => $conditions[2]['value'],
    			':IDTPBUTONAUX'   => $conditions[3]['value'],
    			':NRBUTTONAUX'    => $conditions[4]['value'],
    			':NRPGCONFTAUX'   => $conditions[5]['value'],
    			':IDTPBUTONAUX2'  => $conditions[6]['value'],
    			':NRBUTTONAUX2'   => $conditions[7]['value'],
    			':NRPGCONFTELA'   => $conditions[8]['value'],
    			':CDGRUPO'        => $conditions[9]['value'],
    			':PROPRIAPOSICAO' => $conditions[10]['value'],
    			':NRPGCONFTAUX2'   => $conditions[11]['value']
    			
    		);
            if( empty($params[':IDTPBUTONAUX2']) )
                $params[':IDTPBUTONAUX2'] = "0";
            
            if( empty($params[':NRBUTTONAUX2']) )
                $params[':NRBUTTONAUX2'] = "0";
                
            if( empty($params[':NRPGCONFTAUX2']) )
                $params[':NRPGCONFTAUX2'] = "0";
                
            if( empty($params[':CDGRUPO']) )
                $params[':CDGRUPO'] = "0";
            
            $resultado = $this->layoutService->findPosicaoDisponivelBD($params);
            
            $posicoes = array("01", "02", "03", "04", "05", "06", "07", "08", "09", "10", "11", "12", "13",
                              "14", "15", "16", "17", "18", "19", "20", "21", "22", "23", "24", "25", "26",
                              "27", "28", "29", "30", "31", "32", "33", "34", "35", "36");
            foreach ($resultado as $posicao) {
                for($i = 0;  $i <= sizeof($posicoes) - 1 ; $i++) {
                    if($posicao['NRBUTTON'] == $posicoes[$i]) {
                        array_splice($posicoes, $i, 1);
                    }
                }
            }
            
            foreach($posicoes as $pos) {
                array_push($retorno, ["NRBUTTON" => $pos]);
            }
        }
        
        $response->addDataSet(new DataSet($this->dataSourceName, $retorno));
    }
    
    public function deleteRows (DTO\Request\DataSet $request, DTO\Response $response) { 
        $this->dataSourceName = $this->dataSourceNameArray['filterGrupoSubProduto'];
        parent::delete($request,$response);
    }
    
    public function delete(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceName);
            $deletedRows = $this->dataSourceManager->delete($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceName, $deletedRows));
            $nrDeletedRows = count($deletedRows);
            
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function deleteRecDataSet (DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterLayoutRec'];
        parent::delete($request,$response);
    }
    
    public function findFamilia(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['findFamilia'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['filterFamilia'];
        parent::find($request, $response);
    }
    
    public function findModulos(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['findModulos'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['filterModulos'];
        parent::find($request, $response);
    }
    
    private function setValuesProductsAuyo ($row){
        return  array(
                'CDPRODUTO'=>$row['CDPRODUTO'],
                'CDARVPROD'=>$row['CDARVPROD'],
                'DSBUTTON'=>$row['NMPRODUTO']
            );
    }
    
    public function getProdutosCadastroAutomatico ($row){
        
        
        if($row['INTERVALISTA'] == 'L'){
            $product = array();
            foreach($row['CDPRODUTO'] as $key => $prod){
                $newProd['CDPRODUTO'] = $row['CDPRODUTO'][$key];
                $newProd['CDARVPROD'] = $row['CDARVPRODL'][$key];
                $newProd['DSBUTTON']  = $row['NMPRODUTOL'][$key];
                array_push($product, $newProd);
            }
        } else if($row['INTERVALISTA'] == 'I'){
            if(empty($row['CDARVPRODINI']) || empty($row['CDARVPRODFINAL'])) {
                $product = $this->layoutService->getIntervalProdAll($row);
    		} else {
    		    $product = $this->layoutService->getIntervalProd($row);
    		}
        } else if($row['INTERVALISTA'] == 'D'){
             $product = $this->layoutService->getIntervalProdNivel($row);
        }
		return $product;
    }
    
    /*
        Alguns parâmetros são fixos, por isso tem alguns números sendo setados no 
        inicio da função
    */
    public function saveCadAut(DTO\Request $request, DTO\Response $response) {
        $row = $request->getDataSet()->getRows()[0];
		$prodArray = $this->getProdutosCadastroAutomatico($row);
        $registrossalvos = 0;
        $quantidadeProdutos = count($prodArray);
        
        if(empty($row['NRCOLORBACK']))
            $row['NRCOLORBACK'] = 8421504;
        if(empty($row['NRCOLORTEXT']))
            $row['NRCOLORTEXT'] = 0; 

		$row['NRPGCONFTAUX'] = $row['NRPGCONFTELA'];
		$row['NRORGINCLUSAO'] = $row['NRORG'];
        $row['NRFONTETEXT'] = '08';
		$row['IDTPBUTONAUX'] = '2';
		$row['IDTPBUTTON'] = '1';
		$row['IDTPBUTONAUX'] = '2';
		$row['IDTPBUTONAUX2'] = '0';
		$row['NRBUTTONAUX2'] = '0';
		$row['NRPGCONFTAUX2'] = '0';
		$row['NRPGCONFTELA'] = 1; // Isso serve para o backend começar a procurar a partir da primeira tela
		$row['__is_new'] = true;
		if(count($prodArray)){
		    foreach($prodArray as $product) {
                $posEPag = $this->getNextPosition($row);
                
                if(is_array($posEPag) && count($posEPag) > 0) {
                    $row['NRBUTTON']      = str_pad($posEPag['NRBUTTON'], 0,2, STR_PAD_LEFT);
                    $row['CDPRODUTO']     = $product['CDPRODUTO'];
                    $row['CDARVPROD']     = $product['CDARVPROD'];
                    $row['NRPGCONFTELA']  = ((int) $posEPag['PGCONFTELA']);
                    $row['DSBUTTON']      = $product['DSBUTTON'];
                    $row['CDIDENTBUTON']  = $product['CDPRODUTO'];
                    self::saveDefaulLayout($row, $response, $this->dataSourceNameArray['saveProdCadAut']);
                    $registrossalvos++;
                    //$row['NRPGCONFTELA'] = $posEPag['PGCONFTELA'] + 1; // Isso melhora a performace. Diminui a qnt de pg pesquisadas
                } else {
                    break;
                }
            }
            if($registrossalvos == $quantidadeProdutos) {
                $response->addNotification(new Notification("Todos os produtos foram cadastrados com sucesso.", Notification::TYPE_SUCCESS));
            } else if($registrossalvos == 0) {
                $response->addMessage(new Message("Não existe posições disponíveis no grupo de produto selecionado.", "E"));
            } else {
                $response->addMessage(new Message("Não foi possível cadastrar todos os produtos. Produtos cadastrados:". $registrossalvos, "W"));
            }
		}else{
		    $response->addMessage(new Message("Não foi encontrado produtos para serem cadastrados.", "M"));
		}
    }
    
    /*
        Obtem os produtos dentro de um intervalo de produto inicial e final.
    */
    
    public function getProdutosSaveCadastroAutomatico($row) {
        $params = array( 
			':NRORG'          => $row['NRORG'],
			':CDFILIAL'       => $row['CDFILIAL']
		);
		
		if(empty($row['CDFAMILISALD'])) {
		    $params[':TCDFAMILISALD'] = 'T';
		    $params[':CDFAMILISALD'] = 'T';
		} else {
		    $params[':CDFAMILISALD'] = $row['CDFAMILISALD'];
		    $params[':TCDFAMILISALD'] = null;
		}
        			
		if(empty($row['CDARVPRODINI']) || empty($row['CDARVPRODFINAL'])) {
		    $prodArray = $this->layoutService->GetAllProd($params);
		} else {
			$params[':CDARVPRODINI'] = $row['CDARVPRODINI'];
			$params[':CDARVPRODFINAL'] = $row['CDARVPRODFINAL'];
            $prodArray = $this->layoutService->GetProdInterval($params);
		}
        return $prodArray;
    }
    
    /*
        Obtem o próximo conjunto array = [pagina, posição] a ser inserido no cadastro automático.
        Essa função analisa se será necessário criar um navegador ou não, pois não se pode inserir
        grupos de produtos na posição 1 e nem na posição 36 de uma determinada página.
    */
    public function getNextPosition($row) {
        $linha = $row;
        $params = array( 
			':NRORG'          => $linha['NRORG'],
			':CDFILIAL'       => $linha['CDFILIAL'],
			':NRCONFTELA'     => $linha['NRCONFTELA'],
			':NRPGCONFTELA'   => $linha['NRPGCONFTELA'],
			':NRPGCONFTAUX'   => $linha['NRPGCONFTAUX'],
			':NRBUTTONAUX'    => $linha['NRBUTTONAUX'],
			':CDLOJA'         => $linha['CDLOJA'],
			':DTINIVIGENCIA'  => $linha['DTINIVIGENCIA']
		);

		// Primeiro parâmetro é a página e o segundo é a posição
		$posEPag = $this->layoutService->encontraProximaPosicao($params);
		/* Confere se a posição é igual a posição reservada para o navegador 
		   Se a pos do botão é a última e ele não está na última pág
		   Se a pos do botão é a primeira e ele não está na primeira pág
		*/
		
		
		if(is_array($posEPag) && count($posEPag) > 0) {
    		if(($posEPag['PGCONFTELA'] != 1 && $posEPag['NRBUTTON'] == "01") || ($posEPag['PGCONFTELA'] != 10 && $posEPag['NRBUTTON'] == "36")) {
    	        $linha['NRBUTTON']      = $posEPag['NRBUTTON'];
                $linha['NRPGCONFTELA']  = $posEPag['PGCONFTELA'];
                $pagOrpos = null;
                
    	        if($posEPag['NRBUTTON'] == '01') {
    	            $linha['IDTPBUTTON'] = "A";
    	            $linha['DSBUTTON'] = "Navegador Esquerda";
    	            $pagOrpos = 'pos';
    	        } else if ($posEPag['NRBUTTON'] == 36) {
    	            $linha['IDTPBUTTON'] = "9";
    	            $linha['DSBUTTON'] = "Navegador Direita";
    	            $pagOrpos = 'pag';
    	        }
    	        
    	        $old['NRCOLORBACK'] = $row['NRCOLORBACK'];
    	        $old['NRCOLORTEXT'] = $row['NRCOLORTEXT'];
    	        
    	        $linha['NRCOLORBACK']   = 8421504; // Cor da cinza
    	        $linha['NRCOLORTEXT']   = 0; // Cor preta.
    	        $linha['CDPRODUTO']     = " ";
                $linha['CDIDENTBUTON']  = " ";
                $linha['CDARVPROD']     = " ";
                
                $novoResponse = new DTO\Response;
    		    self::saveDefaulLayout($linha, $novoResponse, $this->dataSourceNameArray['saveProdCadAut']);
    		    if($pagOrpos == 'pos'){
    		        $linha['NRBUTTON']      = '02';
    		    } else if($pagOrpos == 'pag'){
                    $linha['NRPGCONFTELA']  = $posEPag['PGCONFTELA'] + 1;
    		    }
    		    
    		    $posEPag = $this->getNextPosition($linha);
    		
    		    $row['NRCOLORBACK'] = $old['NRCOLORBACK'];    
    		    $row['NRCOLORTEXT'] = $old['NRCOLORTEXT'];
    		    $row['IDTPBUTTON'] = "1";
    		}
		}
		
		return $posEPag;
    }
    /* Função do cadastro automático */
    public function findCadAut (DTO\Request\Filter $request, DTO\Response $response) {
        $response = [];
    }
    
    public function findGrupoProdutoCadAut (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterGrupoProdutoCadAut'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['filterGrupoProdCadAut'];
        parent::find($request, $response);
    }
    
    public function findProdutosCadAut(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterProdutosCadAud'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['filterProdutosCadAud'];
        parent::find($request, $response);
    }
    
    public function duplicateCaixaConfig (DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        
        $row["NRNOVO"] = $this->layoutService->getNovoCod(array(
            ":P_CDFILIAL"       => $row["CDFILIAL"],
            ":NRORG"            => $row["NRORG"]
        ))[0]["NRCONFTELA"] + 1;
    
        $row["NRNOVO"] = $this->appendZeros($row["NRNOVO"]);
        
        $this->layoutService->insertNovaConfiguracao(array(
            ":P_NRNOVO"         => $row["NRNOVO"],
            ":P_NMCONFTELA"     => $row["destConfig"],
            ":P_CDFILIAL"       => $row["CDFILIAL"],
            ":P_NRCONFTELA"     => $row["NRCONFTELA"],
            ":DTINIVIGENCIA"    => $row["DTINIVIGENCIA"],
            ":NRORG"            => $row["NRORG"]
        ));
        
        $this->layoutService->insertItensNovaConfiguracao(array(
            ":P_NRNOVO"         => $row["NRNOVO"],
            ":P_CDFILIAL"       => $row["CDFILIAL"],
            ":P_NRCONFTELA"     => $row["NRCONFTELA"],
            ":DTINIVIGENCIA"    => $row["DTINIVIGENCIA"],
            ":NRORG"            => $row["NRORG"]
        ));
        
    }
    
    public function appendZeros ($cod) {
        return str_pad($cod, 5, "0", STR_PAD_LEFT);
    }
    
    public function removeLayout(DTO\Request\Row $request, DTO\Response $response) {
        try{
            $row = $request->getRow();
            $this->layoutService->removeFilhos($row);
            $response->addDataSet(new DataSet($this->dataSourceNameArray['filterLayout'], array($row)));
        }catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function findVendaItemProdutos (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['findVendaItemProdutos'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['filterProdutosSugestaoItem'];
        parent::find($request, $response);
    }
    
    public function changeGroup (DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        foreach ($row as $value) {
            if($value["OCUPARPOSICAO"] == "S") {
                $posicoes = self::findPosDisponivel($value);
                $value["NEXTNRBUTTONSON"] = $posicoes[0]["NRBUTTON"];
                $this->layoutService->changeGroup($value);
            } else {
                $proxPosBd = $this->layoutService->getNextButton($value);
                $proxPos = (string)$proxPosBd[0]["NEXTBUTTON"];
                $proxPos = str_pad($proxPos, 2, "0", STR_PAD_LEFT);
                if ($proxPos >= 37) {
                    $value["NEXTPGSON"] += 1;
                    $value["NEXTPGSON"] = (string)$value["NEXTPGSON"];
                    $proxPosBd = $this->layoutService->getNextButton($value);
                    $proxPos = (string)$proxPosBd[0]["NEXTBUTTON"];
                    $proxPos = str_pad($proxPos, 2, "0", STR_PAD_LEFT);
                    $value["NEXTNRBUTTONSON"] = $proxPos;
                    if($proxPos == null || $proxPos == "00" || $proxPos == "") {
                        $value["NEXTNRBUTTONSON"] = "01";
                    }
                    $this->layoutService->changeGroup($value);
                } else {
                    $value["NEXTNRBUTTONSON"] = $proxPos;
                    if($proxPos == null || $proxPos == "00" || $proxPos == "") {
                        $value["NEXTNRBUTTONSON"] = "01";
                    }
                    $this->layoutService->changeGroup($value);
                }
            }
        }
    }
    
    public function findPosDisponivel (&$row) {
        $retorno = [];
        for ($i = 0; $i < 10; $i++) {
            if(count($row) > 0) {
                $params = array(
        			':NRORG'  		  => $row["NRORG"],
        			':CDFILIAL'       => $row["CDFILIAL"],
        			':NRCONFTELA'     => $row["NRCONFTELA"],
        			':IDTPBUTONAUX'   => '2',
        			':NRBUTTONAUX'    => $row["NEXTNRBUTTONAUX"],
        			':NRPGCONFTAUX'   => $row["NEXTPGAUX"],
        			':IDTPBUTONAUX2'  => '0',
        			':NRBUTTONAUX2'   => '0',
        			':NRPGCONFTELA'   => $row["NEXTPGSON"],
        			':CDGRUPO'        => '0',
        			':PROPRIAPOSICAO' => '0',
        			':NRPGCONFTAUX2'   => '0'
        		);
                $resultado = $this->layoutService->findPosicaoDisponivelBDProduto($params);
                
                $posicoes = array("01", "02", "03", "04", "05", "06", "07", "08", "09", "10", "11", "12", "13",
                                  "14", "15", "16", "17", "18", "19", "20", "21", "22", "23", "24", "25", "26",
                                  "27", "28", "29", "30", "31", "32", "33", "34", "35", "36");
                                  
                foreach ($resultado as $posicao) {
                    for($j = 0;  $j <= sizeof($posicoes) - 1 ; $j++) {
                        if($posicao['NRBUTTON'] == $posicoes[$j]) {
                            array_splice($posicoes, $j, 1);
                        }
                    }
                }
                
                foreach($posicoes as $pos) {
                    array_push($retorno, ["NRBUTTON" => $pos]);
                }
                
                if(count($retorno) == 0) {
                    $row["NEXTPGSON"] = $row["NEXTPGSON"] + 1;
                    $row["NEXTPGSON"] = (string)$row["NEXTPGSON"];
                }else{
                    $i = 11;
                }
            }
        }  
        return $retorno;
    }
    
    public function updateProductsAndSubGroupsColor(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
            
        $products = $this->layoutService->getProductsAndSubGroupsByColor($row);
        
        $failedProducts = [];
        
        foreach($products as $product) {
            $product['NRCOLORTEXT'] = $row['NRCOLORTEXT'];
            $product['NRCOLORBACK'] = $row['NRCOLORBACK'];
            
            try {
                $this->layoutService->updateProductsAndSubGroupsColor($product);
            } catch(\Exception $e) {
                array_push($failedProducts, $e->getMessage()); 
            }
        }
        
        if(count($failedProducts) > 0) {
            $response->addMessage(new Message("Nem todos os produtos foram alterados. Seguem os Erros: " . implode($failedProducts, "\n"), "E"));
        }
    }
    
    public function checkSubGrupoProdutos(DTO\Request\Row $request, DTO\Response $response){
        $row = $request->getRow();
        $subGrupos = $this->layoutService->getCountSubGrupoGrupo($row);
        $produtos = $this->layoutService->getCountProdutosGrupo($row);
        $retorno = array();
        $retorno['subgrupo'] = true;
        $retorno['produtos'] = true;
        
        if(empty($subGrupos)){
            $subGrupos[0]['TOTAL'] = 0;
        }
        if(empty($produtos)){
            $produtos[0]['TOTAL'] = 0;
        }
        
        if($subGrupos[0]['TOTAL'] > 0 && $produtos[0]['TOTAL'] == 0){
            $retorno['subgrupo'] = true;
            $retorno['produtos'] = false;
        }else if($subGrupos[0]['TOTAL'] == 0 && $produtos[0]['TOTAL'] > 0){
            $retorno['subgrupo'] = false;
            $retorno['produtos'] = true;
        }else if ($subGrupos[0]['TOTAL'] == 0 && $produtos[0]['TOTAL'] == 0){
            $retorno['subgrupo'] = true;
            $retorno['produtos'] = true;
        }
        
        $response->addDataSet(new DataSet('resultado', $retorno));
    }
}