<?php

namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\Report\ReportService;
use Teknisa\MAN\Controllers;
use Teknisa\MAN\Services;
use \Zeedhi\Framework\Util\Functions;

class RelMovimentacaoCaixa extends CrudWithParameterBag {

    const RMC   = 'rmc',
          RFC   = 'rfc',
          RMCNS = 'rmcns',
          RFCECF= 'rfc_ecf',
          RFCPL = 'RFC_PorLoja';


    protected $relatorio;
    protected $birtReport;
    protected $dataSourceName = 'vnd06600_relMovimentacaoCaixa_abertura';
	protected $parameterBagColumns = array('CDFILIAL', 'CDCAIXA', 'NRORG', 'DATAINI');

    public function __construct (ReportService $reportService, Services\Relatorio $relatorio, BirtReport $birtReport) {
        $this->reportService = $reportService;
        $this->relatorio = $relatorio;
        $this->birtReport = $birtReport;
    }

    public function geraRelatorio(DTO\Request\Row $request, DTO\Response $response) {

        $row = $request->getRow();

        // Case there's no CDLOJA in the request row,
        // Gets and sets all CDLOJA from selected FILIAIS

        if(!Functions::arrayKeyExists('CDCAIXA', $row) ||
            (is_array($row['CDCAIXA']) && count($row['CDCAIXA']) === 0) ||
              ($row['CDCAIXA'] == null)) {
            // Do the same for Loja

            if(!array_key_exists('CDLOJA', $row) ||
                (is_array($row['CDLOJA']) && count($row['CDLOJA']) === 0) ||
                 ($row['CDLOJA'] == null)) {
                $resultLojas = $this->relatorio->getAllLojasByFilial($row['CDFILIAL'], $row['NRORG']);

                $row['CDLOJA'] = array();
                foreach($resultLojas as $loja) {
                    array_push( $row['CDLOJA'], $loja['CDLOJA']);
                }
            }

            $resultCaixas = $this->relatorio->getAllCaixasByFilial($row['CDFILIAL'], $row['CDLOJA'], $row['NRORG']);

            $row['CDCAIXA'] = array();
            foreach($resultCaixas as $caixa) {
                array_push( $row['CDCAIXA'], $caixa['FILICAIXA']);
            }
        }

        $parameterMapping = array (
            "CDFILIAL" => "CDFILIAL",
            "CDCAIXA" => "CDCAIXA",
            "NRORG" => "NRORG"
        );

        if(($row['__report_name'] == RelMovimentacaoCaixa::RMC) || ($row['__report_name'] == RelMovimentacaoCaixa::RMCNS)) {

            $parameterMapping = array_merge(
                $parameterMapping,
                array(
                    "DATA" => "DATA",
                    "DATES" => "DATES",
                    "FILIAIS" => "FILIAIS",
                    "DATA_REPEATER" => "DATA_REPEATER",
                    "FILIAL_REPEATER" => "FILIAL_REPEATER",
                    "CAIXA_REPEATER" => "CAIXA_REPEATER",
                    "CAIXA_POSICAO" => "CAIXA_POSICAO"
                )
            );
            if(!$row['CDCAIXA'])
                $row['CDCAIXA'] = array();
            sort($row['CDCAIXA']);
            $row['DATA_REPEATER'] = count($row['DATES']);
            $row['FILIAL_REPEATER'] = count($row['CDFILIAL']);
            $row['CAIXA_REPEATER'] = ceil(count($row['CDCAIXA'])/5);
            $row['CDCAIXA'] = "'".implode(',', $row['CDCAIXA'])."'";
            $row['DATES'] = implode(',', $row['DATES']);
            $row['FILIAIS'] = implode(',', $row['CDFILIAL']);
            $row['CDFILIAL'] = '1';
            $row['DATA'] = '1';
            $row['CAIXA_POSICAO'] = 1;

            $this->generateURLByReport($row, $parameterMapping, $response);

        }
        else if(($row['__report_name'] == RelMovimentacaoCaixa::RFC) || ($row['__report_name'] == RelMovimentacaoCaixa::RFCECF)) {
            $parameterMapping = array_merge(
                $parameterMapping,
                array(
                    "DATAINICIO"  => "DATAINICIO",
                    "DATAFINAL"   => "DATAFINAL",
                    "DTENTRVENDA" => "DTENTRVENDA",
                    "DTABERTUR"   => "DTABERTUR",
                    "TDTABERTUR"  => "TDTABERTUR"
                )
            );

            if(!$row['CDCAIXA'])
                $row['CDCAIXA'] = array();
            sort($row['CDCAIXA']);
            $row['CDFILIAL'] = implode(',', $row['CDFILIAL']);
            $row['CDCAIXA'] = implode(',', $row['CDCAIXA']);
            $row['DATAINICIO'] = $row['DTENTRVENDA']['start'];
            $row['DATAFINAL'] = $row['DTENTRVENDA']['end'];
            $row['TDTABERTUR'] = 'NA';
            $row['DTABERTUR'] = 'NA';

            if(empty($row['BIRTABRTFORMAT']))
                $row['TDTABERTUR'] = 'T';
            else
                $row['DTABERTUR'] = implode('_', $row['BIRTABRTFORMAT']);

            $row['DTENTRVENDA'] = '1';

            $this->generateURLByReport($row, $parameterMapping, $response);
        } else if($row['__report_name'] == RelMovimentacaoCaixa::RFCPL) {

            $paramUnidades = isset($row['CDFILIAL']) ? implode('_', $row['CDFILIAL']) : [];
            if(empty($paramUnidades)) {
                $paramUnidades = 'N';
                $TparamUnidades = 'T';
            } else {
                $TparamUnidades = 'N';
            }

            $paramLojas = isset($row['FILILOJA']) ? implode('_', $row['FILILOJA']) : [];
            if(empty($paramLojas)) {
                $paramLojas = 'N';
                $TparamLojas = 'T';
            } else {
                $TparamLojas = 'N';
            }

            $row['CDFILIAL'] = $paramUnidades;
            $row['TCDFILIAL'] = $TparamUnidades;
            $row['CDLOJA'] = $paramLojas;
            $row['TCDLOJA'] = $TparamLojas;
            $row['DTINIVEND'] = $row['DTENTRVENDA']['start'];
            $row['DTFINVEND'] = $row['DTENTRVENDA']['end'];

            if(isset($row['NMLOJA'])){
                unset($row['NMLOJA']);
            }
            if(isset($row['FILILOJA'])){
                unset($row['FILILOJA']);
            }
            unset($row['DTENTRVENDA']);
            unset($row['NMFILIAL']);
            unset($row['CDCAIXA']);
            unset($row['DATES']);
            $parameterMapping = array_merge(
                $parameterMapping,
                array(
                    "CDFILIAL" => "P_CDFILIAL",
                    "DTINIVEND"  => "P_DTINIVEND",
                    "DTFINVEND"   => "P_DTFINVEND",
                    "CDLOJA" => "P_CDLOJA",
                    "TCDLOJA" => "P_TCDLOJA",
                    "TCDFILIAL" => "P_TCDFILIAL"
                )
            );
            $this->generateURLByReport($row, $parameterMapping, $response);
        } else {
            $response->setError(new Error('Erro ao encontrar o relatório', 500));
            $chamaRelatorio = false;
        }

    }

    private function generateURLByReport($row, $parameterMapping, $response) {
        try {
            $relatorioData = $this->relatorio->buildParametersByRelatorio($row, $parameterMapping);
            // $birtURL = $this->birtReport->createRemoteReport($request, $response);
            $birtURL = $this->reportService->createRemoteReport('BIRT', $relatorioData['ReportName'], $relatorioData['ParametersRow']);
            $response->addDataset(new DataSet('URL',array($birtURL)));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage() , 500));
        }
    }

    public function findAberturas(DTO\Request\Filter $request, DTO\Response $response) {
        $params = $request->getFilterCriteria()->getConditions();
        $ALLFIEDLS[':ALLFIEDLS'] = 'T';
        $CDCAIXA = $TCDCAIXA = null;
        $CDLOJA = $TCDLOJA = null;
        
        foreach ($params as $filtro) {
            if($filtro['columnName'] == 'CDFILIAL') {
                if($filtro['value'] == null)
                    $CDFILIAL = 'T';
                else
                    $CDFILIAL = implode('_', $filtro['value']);
            } 
            
            if ($filtro['columnName'] == 'CDCAIXA') {
                if($filtro['value'] == null)
                    $TCDCAIXA = 'T';
                if ($filtro['value'] !== null){
                    if(count($filtro['value']) == 1){
                        $aux =  $filtro['value'][0];
                        $caixa = explode('_', $aux);
                        $CDCAIXA = $caixa[0];
                    }
                    else
                        $CDCAIXA = implode('_', $filtro['value']);
                }
            } 
            
            if ($filtro['columnName'] == 'CDLOJA') {
                if($filtro['value'] == null)
                    $TCDLOJA = 'T';
                else
                    $CDLOJA = implode('_', $filtro['value']);
            } 
            
            if ($filtro['columnName'] == 'NRORG') {
                $NRORG = $filtro['value'];
            } 
            
            if ($filtro['columnName'] == 'DATAINI') {
                $DATAINI = $filtro['value']['start'];
                $DATAFIN = $filtro['value']['end'];
            }

            if(strpos($filtro['columnName'], '|')){
                $colunas = explode('|', $filtro['columnName']);
                foreach($colunas as $k => $con){
                    $ALLFIEDLS[':ALLFIEDLS'] = $filtro['value'][0]['_ALL'];
                }
            }
        }

        $params = array(
            ':CDFILIAL'  => $CDFILIAL,
            ':CDLOJA'    => $CDLOJA,
            ':TCDLOJA'   => $TCDLOJA,
            ':CDCAIXA'   => $CDCAIXA,
            ':TCDCAIXA'  => $TCDCAIXA,
            ':DATAINI'   => $DATAINI,
            ':DATAFIN'   => $DATAFIN,
            ':NRORG'     => $NRORG,
            ":ALLFIEDLS" =>  mb_strtoupper($ALLFIEDLS[':ALLFIEDLS'], 'UTF-8')
        );
        $aberturas = $this->relatorio->getAberturas($params);
        $response->addDataSet(new DataSet($this->dataSourceName, $aberturas));
    }

}