<?php 

namespace Teknisa\MAN\Services;

use Teknisa\MAN\Utilities;
use \Doctrine\ORM\EntityManager;


class ProdutividadeVendedor {
    
	protected $entityManager; 

	public function __construct(EntityManager $entityManager)  {
		$this->entityManager = $entityManager;
	}

    private function setReportParameters(&$params, &$conditions) {
        $params = array(
            ':CDLOJA' => isset($conditions['CDLOJA']) ? $conditions['CDLOJA'] : null ,
            ':TCDLOJA' => isset($conditions['TCDLOJA']) ? $conditions['TCDLOJA'] : null ,
            ':DATAINI' => isset($conditions['DATAINI']) ? $conditions['DATAINI'] : null ,
            ':DATAFIN' => isset($conditions['DATAFIN']) ? $conditions['DATAFIN'] : null ,
            ':CDFILIAL' => isset($conditions['CDFILIAL']) ? $conditions['CDFILIAL'] : null ,
            ':CDPRODUTO' => isset($conditions['CDPRODUTO']) ? $conditions['CDPRODUTO'] : null ,
            ':TCDFILIAL' => isset($conditions['TCDFILIAL']) ? $conditions['TCDFILIAL'] : null ,
            ':CDVENDEDOR' => isset($conditions['CDVENDEDOR']) ? $conditions['CDVENDEDOR'] : null ,
            ':TCDPRODUTO' => isset($conditions['TCDPRODUTO']) ? $conditions['TCDPRODUTO'] : null ,
            ':TCDVENDEDOR' => isset($conditions['TCDVENDEDOR']) ? $conditions['TCDVENDEDOR'] : null ,
            ':CDPRODUTOAVO' => isset($conditions['CDPRODUTOAVO']) ? $conditions['CDPRODUTOAVO'] : null ,
            ':DTMOVTURCAIX' => isset($conditions['DTMOVTURCAIX']) ? $conditions['DTMOVTURCAIX'] : null ,
            ':TDTMOVTURCAIX' => isset($conditions['TDTMOVTURCAIX']) ? $conditions['TDTMOVTURCAIX'] : null ,
            ':TCDPRODUTOAVO' => isset($conditions['TCDPRODUTOAVO']) ? $conditions['TCDPRODUTOAVO'] : null 
        );
    } 
    
    public function getProdutividadeVendedor($conditions) {
    	$params = [];
        $this->setReportParameters($params, $conditions);
        $query = $this->getQuery($conditions['TIPORELATORIO']);
        $query =  strval(trim($query));
        return $this->entityManager->getConnection()->fetchAll($query, $params);
    }
    
    public function getQueryDadosFilialLojaCaixa(){
        /*
        Tabela DADOS FILIAL/LOJA/CAIXA 
        */   
        $queryFilialLojaCaixa = "SELECT DISTINCT 
        	  C.CDFILIAL, 
        	  C.NMFILIAL, 
        	  C.CDCAIXA, 
        	  C.NMCAIXA, 
        	  L.CDLOJA, 
        	  L.NMLOJA, 
        	  NVL(L.CDPRODTAXASERV, 'X') CDPRODTAXASERV,
        	  NVL(L.CDPRODTAXAENTR, 'X') CDPRODTAXAENTR
         FROM LOJA L,
        	  (SELECT DISTINCT F.CDFILIAL, F.NMFILIAL, CX.CDCAIXA, CX.CDLOJA, CX.NMCAIXA
        		 FROM CAIXA CX,
        			  (SELECT DISTINCT CDFILIAL, NMFILIAL
        				 FROM FILIAL
        				WHERE ((INSTR(:CDFILIAL, CDFILIAL) <> 0) OR (:TCDFILIAL = 'T'))
        			  ) F
        		WHERE CX.CDFILIAL = F.CDFILIAL
        		  AND (INSTR(:DTMOVTURCAIX, '_' ||F.CDFILIAL ||CDCAIXA) > 0 OR :TDTMOVTURCAIX = 'T')
        	  ) C
        WHERE L.CDFILIAL = C.CDFILIAL
          AND L.CDLOJA     = C.CDLOJA
          AND ((INSTR(:CDLOJA, L.CDFILIAL ||L.CDLOJA) <> 0) OR (:TCDLOJA   = 'T'))";
          
        return $queryFilialLojaCaixa;
    }
    
    public function getQueryDadosVendedor(){
        /*
        Tabela V - Busca dados dos vendedores
        */  
        $queryVendedor = "SELECT CDVENDEDOR, NMRAZSOCVEN, NRINSJURVEN
            FROM VENDEDOR
        WHERE ((INSTR(:CDVENDEDOR, CDVENDEDOR) > 0) OR (:TCDVENDEDOR = 'T'))";  
          
        return $queryVendedor;
    }
    
    public function getQueryFILTROVENDA(){
        /*
        Tabela FILTROVENDA - busca dados de venda, repique, e parametrizacao de taxa de servico e taxa de entregue na tabela LOJA , alem do valor de repique por venda
        */    
        $queryFILTROVENDA = "SELECT DISTINCT
        	VE.CDFILIAL,
        	VE.CDCAIXA,
        	VE.CDLOJA,
        	VE.NRSEQVENDA,
        	FILTROGER.CDFILIAL|| VE.CDCAIXA|| TO_CHAR(VE.DTABERTUR, 'DD/MM/YYYY HH24:MI:SS') CHAVE,
        	VE.CDVENDEDOR,
        	FILTROGER.CDPRODTAXASERV,
        	FILTROGER.CDPRODTAXAENTR,
        	FILTROGER.NMLOJA,
        	FILTROGER.NMFILIAL,
        	VE.VRREPIQUEVENDA,
        	VE.DTABERTUR
        FROM
        	VENDA VE,
        	(
        	
        		/*
        		Tabela DADOS FILIAL/LOJA/CAIXA 
        		*/  
        		".$this->getQueryDadosFilialLojaCaixa()."
        		
        	) FILTROGER
        WHERE
        	VE.IDSITUVENDA = 'O'
        	AND VE.CDFILIAL = FILTROGER.CDFILIAL
        	AND VE.CDLOJA = FILTROGER.CDLOJA
        	AND VE.CDCAIXA = FILTROGER.CDCAIXA
        	AND VE.DTENTRVENDA BETWEEN :DATAINI AND :DATAFIN";
          
        return $queryFILTROVENDA;
    }
    
    public function getQueryFILTROREL(){
        /*
        Tabela FILTROREL - com as informacoes de V e FILTROGER, une com os itens de venda, e busca informacoes dos itens em relacao a parametrizacoes 
        */ 
        
        $queryFILTROREL = "SELECT DISTINCT
        	FILTROVENDA.CDFILIAL,
        	FILTROVENDA.NMFILIAL,
        	FILTROVENDA.CDLOJA,
        	FILTROVENDA.NMLOJA,
        	I.CDPRODUTO,
        	NVL(I.CDVENDEDOR, FILTROVENDA.CDVENDEDOR) CDVENDEDOR,
        	FILTROVENDA.NRSEQVENDA,
        	FILTROVENDA.CDPRODTAXASERV,
        	FILTROVENDA.CDPRODTAXAENTR,
        	FILTROVENDA.CDCAIXA,
        	V.NMRAZSOCVEN,
        	V.NRINSJURVEN,
        	FILTROVENDA.VRREPIQUEVENDA,
        	FILTROVENDA.DTABERTUR,
        	TRUNC(I.VRUNITVEND, 2) AS VRUNIT,
        	P.IDCOBTXSERV,
        	SUM(I.QTPRODVEND) QTPRODVEND,
        	SUM(I.VRACRITVEND) ACR,
        	SUM(I.VRDESITVEND) DES,
        	SUM(TRUNC(I.QTPRODVEND *(I.VRUNITVEND + NVL(I.VRUNITVENDCL, 0)), 2) - NVL(I.VRDESITVEND, 0) + NVL(I.VRACRITVEND, 0)) AS VRPRODVENDA
        FROM
        	(
        	
        		/*
        		Tabela V - Busca dados dos vendedores
        		*/         
        		".$this->getQueryDadosVendedor()."
        	
        	) V,
        	(
        	
        		/*
        		Tabela FILTROVENDA - busca dados de venda, repique, e parametrizacao de taxa de servico e taxa de entregue na tabela LOJA , alem do valor de repique por venda
        		*/          
        		".$this->getQueryFILTROVENDA()."
        		
        	) FILTROVENDA,
        	ITEMVENDA   I,
        	PRODUTO     P
        WHERE
        	((INSTR(:DTMOVTURCAIX, FILTROVENDA.CHAVE) > 0 ) OR ( :TDTMOVTURCAIX = 'T' ) )
        	AND I.CDFILIAL = FILTROVENDA.CDFILIAL
        	AND I.CDCAIXA = FILTROVENDA.CDCAIXA
        	AND I.NRSEQVENDA = FILTROVENDA.NRSEQVENDA
        	AND I.CDPRODUTO = P.CDPRODUTO
        	AND NVL(I.CDVENDEDOR, FILTROVENDA.CDVENDEDOR) = V.CDVENDEDOR
        	AND I.CDPRODUTO NOT IN (
        		FILTROVENDA.CDPRODTAXAENTR,
        		FILTROVENDA.CDPRODTAXASERV
        	)
        GROUP BY
        	FILTROVENDA.CDFILIAL,
        	FILTROVENDA.NMFILIAL,
        	FILTROVENDA.CDLOJA,
        	FILTROVENDA.NMLOJA,
        	I.CDPRODUTO,
        	NVL(I.CDVENDEDOR, FILTROVENDA.CDVENDEDOR),
        	FILTROVENDA.NRSEQVENDA,
        	FILTROVENDA.CDPRODTAXASERV,
        	FILTROVENDA.CDPRODTAXAENTR,
        	FILTROVENDA.CDCAIXA,
        	V.NMRAZSOCVEN,
        	V.NRINSJURVEN,
        	FILTROVENDA.VRREPIQUEVENDA,
        	FILTROVENDA.DTABERTUR,
        	TRUNC(I.VRUNITVEND, 2),
        	P.IDCOBTXSERV";
          
        return $queryFILTROREL;
    }
    
    public function getQueryITEMVEND(){
        
        /*
         Tabela ITEMVEND 
        */ 
        $queryITEMVEND = "SELECT V.CDFILIAL, 
        	   V.CDCAIXA, 
        	   V.NRSEQVENDA, 
        	   V.CDLOJA, 
        	   I.VRUNITVEND, 
        	   I.CDPRODUTO,
        	   NVL(I.CDVENDEDOR, V.CDVENDEDOR) CDVENDEDOR, 											
        	   SUM(TRUNC(I.QTPRODVEND * (I.VRUNITVEND + NVL(I.VRUNITVENDCL,0)),2) - NVL(I.VRDESITVEND,0) + NVL(I.VRACRITVEND,0)) VRPRODVEND
        FROM ITEMVENDA I
        JOIN (
        		/*
        		Tabela FILTROVENDA - busca dados de venda, repique, e parametrizacao de taxa de servico e taxa de entregue na tabela LOJA , alem do valor de repique por venda
        		*/  
        		".$this->getQueryFILTROVENDA()."
        
         ) V
        ON I.CDFILIAL = V.CDFILIAL
          AND I.CDCAIXA    = V.CDCAIXA
          AND I.NRSEQVENDA = V.NRSEQVENDA
          AND ((INSTR(:CDPRODUTO, I.CDPRODUTO) > 0) OR (:TCDPRODUTO = 'T'))
        GROUP BY V.CDFILIAL, V.CDCAIXA, V.NRSEQVENDA, V.CDLOJA, I.VRUNITVEND, I.CDPRODUTO, NVL(I.CDVENDEDOR, V.CDVENDEDOR)";
        
          
        return $queryITEMVEND;
    }
    
    public function getQueryDADOSITEM(){
        /*
         Tabela DADOSITEM - calculo dos valores de Acrescimo, Desconto, valor venda, valor unitario, e parametro de taxa de servico por produto 
        */ 
        
        $queryDADOSITEM = "SELECT DISTINCT
        	FILTROREL.CDFILIAL,
        	FILTROREL.NMFILIAL,
        	FILTROREL.CDLOJA,
        	FILTROREL.NMLOJA,
        	FILTROREL.CDPRODUTO,
        	P.CDARVPROD,
        	P.NMPRODUTO,
        	FILTROREL.CDVENDEDOR,
        	FILTROREL.CDPRODTAXASERV,
        	FILTROREL.CDPRODTAXAENTR,
        	FILTROREL.NMRAZSOCVEN,
        	FILTROREL.NRINSJURVEN,
        	FILTROREL.CDCAIXA,
        	FILTROREL.VRREPIQUEVENDA,
        	FILTROREL.QTPRODVEND,
        	FILTROREL.ACR,
        	FILTROREL.DES,
        	FILTROREL.VRPRODVENDA,
        	FILTROREL.NRSEQVENDA,
        	FILTROREL.VRUNIT,
        	FILTROREL.IDCOBTXSERV,                                        
        	P.CDPRODNIVANTE,
			FILTROREL.DTABERTUR
        FROM
        	(
        	   /*
        		Tabela FILTROREL - com as informacoes de V e FILTROGER, une com os itens de venda, e busca informacoes dos itens em relacao a parametrizacoes 
        		*/ 
        		".$this->getQueryFILTROREL()."
        
        	) FILTROREL,
        	(
        		/*
        		Tabela ITEMVEND 
        		*/ 
        		".$this->getQueryITEMVEND()."
        			
        	) ITEMVEND,
        	PRODUTO P
        WHERE
        	ITEMVEND.CDFILIAL = FILTROREL.CDFILIAL
        	AND ITEMVEND.NRSEQVENDA = FILTROREL.NRSEQVENDA
        	AND ITEMVEND.CDLOJA = FILTROREL.CDLOJA
        	AND ITEMVEND.CDVENDEDOR = FILTROREL.CDVENDEDOR
        	AND ITEMVEND.VRUNITVEND = FILTROREL.VRUNIT
        	AND ITEMVEND.CDPRODUTO = FILTROREL.CDPRODUTO
        	AND P.CDPRODUTO = FILTROREL.CDPRODUTO";
        
          
        return $queryDADOSITEM;
    }
    
    public function getQueryVRTXENTREGA(){
        /*
        Tabela VRTXENTREGA - busca valor taxa de entrega
       */ 
       $queryVRTXENTREGA = "SELECT V.CDFILIAL, V.NRSEQVENDA, V.CDCAIXA, SUM(NVL(I.VRACRITVEND,0)) VRTXENTREGA, V.DTABERTUR
          FROM ITEMVENDA I, VENDA V, LOJA L
        WHERE I.CDFILIAL = V.CDFILIAL
           AND I.CDCAIXA    = V.CDCAIXA
           AND I.NRSEQVENDA = V.NRSEQVENDA
           AND V.CDLOJA     = L.CDLOJA
           AND I.CDLOJA     = L.CDLOJA
           AND V.CDFILIAL   = L.CDFILIAL
           AND I.CDPRODUTO IN (L.CDPRODTAXAENTR)
        GROUP BY V.CDFILIAL, V.NRSEQVENDA, V.CDCAIXA, V.DTABERTUR";
        
          
        return $queryVRTXENTREGA;
    }
    
    public function getQueryVALORGERALVENDA(){
        /*
        Tabela VALORGERALVENDA - valores de venda geral de item a item 
       */ 
       $queryVALORGERALVENDA = "SELECT V.CDFILIAL, V.NRSEQVENDA, V.CDCAIXA, 
             SUM(TRUNC(I.QTPRODVEND * (I.VRUNITVEND + NVL(I.VRUNITVENDCL,0)),2) - NVL(I.VRDESITVEND,0) + NVL(I.VRACRITVEND,0)) VRITENS
        FROM ITEMVENDA I, VENDA V, LOJA L
       WHERE ((INSTR(:CDFILIAL, I.CDFILIAL) <> 0) OR (:TCDFILIAL = 'T'))
         AND ((INSTR(:CDFILIAL, V.CDFILIAL) <> 0) OR (:TCDFILIAL = 'T'))
         AND ((INSTR(:CDFILIAL, L.CDFILIAL) <> 0) OR (:TCDFILIAL = 'T')) 
         AND I.CDFILIAL = V.CDFILIAL
         AND I.CDCAIXA    = V.CDCAIXA
         AND I.NRSEQVENDA = V.NRSEQVENDA
         AND V.CDLOJA     = L.CDLOJA
         AND I.CDLOJA     = L.CDLOJA
         AND V.CDFILIAL   = L.CDFILIAL
         AND I.CDPRODUTO <> NVL(L.CDPRODTAXAENTR, 'X')
         AND I.CDPRODUTO <> NVL(L.CDPRODTAXASERV, 'X')
         AND V.DTENTRVENDA BETWEEN :DATAINI AND :DATAFIN
       GROUP BY V.CDFILIAL, V.NRSEQVENDA, V.CDCAIXA";
        
          
        return $queryVALORGERALVENDA;
    }
    
    public function getQueryVALORGERALVENDATAXA(){
        /*
        Tabela VALORGERALVENDATAXA - valores de venda geral de item a item 
       */ 
       $queryVALORGERALVENDATAXA = "SELECT
			V.CDFILIAL,
			V.NRSEQVENDA,
			V.CDCAIXA,
			SUM(TRUNC(I.QTPRODVEND *(I.VRUNITVEND + NVL(I.VRUNITVENDCL, 0)), 2) - NVL(I.VRDESITVEND, 0) + NVL(I.VRACRITVEND, 0)) VRITENSTX
		FROM
			ITEMVENDA   I,
			VENDA       V,
			LOJA        L,
			PRODUTO     P
		WHERE
		    ((INSTR(:CDFILIAL, I.CDFILIAL) <> 0) OR (:TCDFILIAL = 'T'))
            AND ((INSTR(:CDFILIAL, V.CDFILIAL) <> 0) OR (:TCDFILIAL = 'T'))
            AND ((INSTR(:CDFILIAL, L.CDFILIAL) <> 0) OR (:TCDFILIAL = 'T'))
			AND I.CDFILIAL = V.CDFILIAL
			AND I.CDCAIXA = V.CDCAIXA
			AND I.NRSEQVENDA = V.NRSEQVENDA
			AND V.CDLOJA = L.CDLOJA
			AND I.CDLOJA = L.CDLOJA
			AND V.CDFILIAL = L.CDFILIAL
			AND I.CDPRODUTO <> NVL(L.CDPRODTAXAENTR, 'X')
			AND I.CDPRODUTO <> NVL(L.CDPRODTAXASERV, 'X')
			AND P.CDPRODUTO = I.CDPRODUTO
			AND V.DTENTRVENDA BETWEEN :DATAINI AND :DATAFIN
		GROUP BY
			V.CDFILIAL,
			V.NRSEQVENDA,
			V.CDCAIXA";
        
          
        return $queryVALORGERALVENDATAXA;
    }
    
    public function getQueryVALORGORJETA(){
        /*
        Tabela VALORGORJETA - busca valores da gorjeta considerando repique tbem
       */ 
       
       $queryVALORGORJETA = "SELECT V.CDFILIAL, V.CDCAIXA, V.NRSEQVENDA, V.DTABERTUR,
        SUM(NVL(I.VRACRITVEND,0)) AS VRGORJETA
        FROM VENDA V, ITEMVENDA I, LOJA L
        WHERE V.DTENTRVENDA BETWEEN :DATAINI AND :DATAFIN
         AND ((INSTR(:CDFILIAL, V.CDFILIAL) <> 0) OR (:TCDFILIAL = 'T'))
         AND V.IDSITUVENDA = 'O'
         AND V.CDFILIAL    = I.CDFILIAL
         AND V.CDCAIXA     = I.CDCAIXA
         AND V.NRSEQVENDA  = I.NRSEQVENDA
         AND V.CDFILIAL    = L.CDFILIAL
         AND V.CDLOJA      = L.CDLOJA
         AND I.CDPRODUTO   = L.CDPRODTAXASERV
        GROUP BY V.CDFILIAL, V.CDCAIXA, V.NRSEQVENDA, V.DTABERTUR ";
          
        return $queryVALORGORJETA;
    }
    
    public function getQueryPrincipalRelatorio(){
                 
        $queryRELATORIO = "SELECT DISTINCT
                DADOSITEM.CDFILIAL,
                DADOSITEM.NMFILIAL,
                DADOSITEM.CDLOJA,
                DADOSITEM.NMLOJA,
                DADOSITEM.CDARVPROD,
                DADOSITEM.NMPRODUTO,
                DADOSITEM.CDVENDEDOR,
                DADOSITEM.NMRAZSOCVEN,
                DADOSITEM.NRINSJURVEN,
                DADOSITEM.QTPRODVEND,
                DADOSITEM.ACR,
                DADOSITEM.DES,
                DADOSITEM.VRPRODVENDA,
                DADOSITEM.VRUNIT,
                DADOSITEM.VRREPIQUEVENDA,
                DADOSITEM.NRSEQVENDA,
                VRITENS,
                VRITENSTX,
                VRGORJETA,
                VRTXENTREGA,
                DADOSITEM.IDCOBTXSERV,                                
                DADOSITEM.CDPRODUTO,
                DADOSITEM.CDPRODNIVANTE,
			    DADOSITEM.DTABERTUR
            FROM
                (
                    /*
					Tabela DADOSITEM - calculo dos valores de Acrescimo, Desconto, valor venda, valor unitario, e parametro de taxa de servico por produto 
					*/ 
					".$this->getQueryDADOSITEM()."
                ) DADOSITEM
                LEFT JOIN (
					/*
					Tabela VALORGORJETA - busca valores da gorjeta considerando repique tbem
					*/
					".$this->getQueryVALORGORJETA()."
                ) VALORGORJETA ON VALORGORJETA.CDFILIAL = DADOSITEM.CDFILIAL
                                  AND VALORGORJETA.CDCAIXA = DADOSITEM.CDCAIXA
                                  AND VALORGORJETA.NRSEQVENDA = DADOSITEM.NRSEQVENDA
                LEFT JOIN 
                (
					/* 
					Tabela VRTXENTREGA - busca valor taxa de entrega
					*/ 	
					".$this->getQueryVRTXENTREGA()."
                ) VRTXENTREGA ON VRTXENTREGA.CDFILIAL = DADOSITEM.CDFILIAL
                                  AND VRTXENTREGA.CDCAIXA = DADOSITEM.CDCAIXA
                                  AND VRTXENTREGA.NRSEQVENDA = DADOSITEM.NRSEQVENDA,
                (
					/*
					  Tabela VALORGERALVENDA - valores de venda geral de item a item 
					*/ 		
					".$this->getQueryVALORGERALVENDA()."
               ) VALORGERALVENDA,
               (
                     /*
					  Tabela VALORGERALVENDA - valores de venda geral de item a item 
					*/ 		
					".$this->getQueryVALORGERALVENDATAXA()."
                 
               ) VALORGERALVENDATAXA
            WHERE
                VALORGERALVENDA.CDFILIAL = DADOSITEM.CDFILIAL
                AND VALORGERALVENDA.CDCAIXA = DADOSITEM.CDCAIXA
                AND VALORGERALVENDA.NRSEQVENDA = DADOSITEM.NRSEQVENDA
                AND VALORGERALVENDATAXA.CDFILIAL = DADOSITEM.CDFILIAL
                AND VALORGERALVENDATAXA.CDCAIXA = DADOSITEM.CDCAIXA
                AND VALORGERALVENDATAXA.NRSEQVENDA = DADOSITEM.NRSEQVENDA";
        
        
        return $queryRELATORIO;
    }
    
    public function getQueryPrincipalTOTAL(){
        
        $queryPrincipalTOTAL = "
            SELECT
                (CDFILIAL|| ' - '|| NMFILIAL )|| ' - '||(CDLOJA|| ' - '|| NMLOJA ) AS LABELFILIAL,
                (CDVENDEDOR|| ' - '|| NMRAZSOCVEN) AS GARCON,
                FINAL.*,
                (FINAL.VRPRODVENDA + FINAL.VRREPITEM + FINAL.VRGORJETA + FINAL.VRTXENTREGA) VRFINALITEM,
                (FINAL.VRPRODVENDA + FINAL.VRGORJETA + FINAL.VRTXENTREGA) VRLIQUIDO
            FROM
                (
                    SELECT
                        CONSOLIDADO.CDFILIAL,
                        CONSOLIDADO.NMFILIAL,
                        CONSOLIDADO.CDLOJA,
                        CONSOLIDADO.NMLOJA,
                        CONSOLIDADO.CDARVPROD,
                        CONSOLIDADO.NMPRODUTO,
                        CONSOLIDADO.CDVENDEDOR,
                        CONSOLIDADO.NMRAZSOCVEN,
                        CONSOLIDADO.NRINSJURVEN,
                        CONSOLIDADO.VRUNIT,
                        NVL(SUM(((VRGORJETAVENDA - VRREPIQUEVENDA) * DECODE(IDCOBTXSERV, 'S', VRPRODVENDA, 0)) / VRGERALITEMVENDATX), 0) VRGORJETA,
                        NVL(SUM(((VRTXENTREGAVENDA - VRREPIQUEVENDA) * DECODE(IDCOBTXSERV, 'S', VRPRODVENDA, 0)) / VRGERALITEMVENDATX), 0) VRTXENTREGA,
                        SUM(QTPRODVEND) QTPRODVEND,
                        SUM(ACR) ACR,
                        SUM(DES) DES,
                        SUM(VRPRODVENDA) VRPRODVENDA,
                        SUM(VRREPITEMTEMP / VRGERALITEMVENDA) AS VRREPITEM,
                        NVL(SUM(VRREPIQUEVENDA), 0) AS VRREPIQUEVENDA,               
                        CONSOLIDADO.CDPRODUTO,
                        CONSOLIDADO.CDPRODNIVANTE,
                        CONSOLIDADO.DTABERTUR
                    FROM
                        (
                            SELECT
                                RELATORIO.CDFILIAL,
                                RELATORIO.NMFILIAL,
                                RELATORIO.CDLOJA,
                                RELATORIO.NMLOJA,
                                RELATORIO.CDARVPROD,
                                RELATORIO.NMPRODUTO,
                                RELATORIO.CDVENDEDOR,
                                RELATORIO.NMRAZSOCVEN,
                                RELATORIO.NRINSJURVEN,
                                RELATORIO.VRUNIT,
                                RELATORIO.NRSEQVENDA,
                                RELATORIO.IDCOBTXSERV,
                                NVL(RELATORIO.VRREPIQUEVENDA, 0) VRREPIQUEVENDA,
                                SUM(RELATORIO.QTPRODVEND) QTPRODVEND,
                                SUM(RELATORIO.ACR) ACR,
                                SUM(RELATORIO.DES) DES,
                                SUM(RELATORIO.VRPRODVENDA) VRPRODVENDA,
                                SUM(RELATORIO.VRGORJETA) VRGORJETAVENDA,
                                SUM(RELATORIO.VRTXENTREGA) VRTXENTREGAVENDA,
                                NVL(SUM(RELATORIO.VRREPIQUEVENDA * RELATORIO.VRPRODVENDA), 0) VRREPITEMTEMP,
                                SUM(RELATORIO.VRITENS) VRGERALITEMVENDA,                        
                                SUM(RELATORIO.VRITENSTX) VRGERALITEMVENDATX,                        
                                RELATORIO.CDPRODUTO,
                                RELATORIO.CDPRODNIVANTE,
                                RELATORIO.DTABERTUR
                            FROM
                                (						
        						 /*
        						  Tabela RELATORIO 
        						 */
        						 ".$this->getQueryPrincipalRelatorio()."
                                ) RELATORIO
                            GROUP BY
                                RELATORIO.CDFILIAL,
                                RELATORIO.NMFILIAL,
                                RELATORIO.CDLOJA,
                                RELATORIO.NMLOJA,
                                RELATORIO.CDARVPROD,
                                RELATORIO.NMPRODUTO,
                                RELATORIO.CDVENDEDOR,
                                RELATORIO.NMRAZSOCVEN,
                                RELATORIO.NRINSJURVEN,
                                RELATORIO.VRUNIT,
                                RELATORIO.NRSEQVENDA,
                                NVL(RELATORIO.VRREPIQUEVENDA, 0),
                                RELATORIO.IDCOBTXSERV,                       
                                RELATORIO.CDPRODUTO,
                                RELATORIO.CDPRODNIVANTE,
                                RELATORIO.DTABERTUR
                        ) CONSOLIDADO
                    GROUP BY
                        CONSOLIDADO.CDFILIAL,
                        CONSOLIDADO.NMFILIAL,
                        CONSOLIDADO.CDLOJA,
                        CONSOLIDADO.NMLOJA,
                        CONSOLIDADO.CDARVPROD,
                        CONSOLIDADO.NMPRODUTO,
                        CONSOLIDADO.CDVENDEDOR,
                        CONSOLIDADO.NMRAZSOCVEN,
                        CONSOLIDADO.NRINSJURVEN,
                        CONSOLIDADO.VRUNIT,
                        CONSOLIDADO.CDPRODUTO,
                        CONSOLIDADO.CDPRODNIVANTE,
                        CONSOLIDADO.DTABERTUR) FINAL";
        
        return $queryPrincipalTOTAL;
        
    }
    
    public function getQuery($tipoRelatorio){
        
        $query = "
            SELECT 
            	TOTAL.*,
            	PP.CDARVPROD AS CDARVPRODPAI,
            	PPP.CDARVPROD AS CDARVPRODAVO,
            	PP.NMPRODUTO AS NMPRODUTOPAI,
            	PPP.NMPRODUTO AS NMPRODUTOAVO         
            FROM( 
                 /*
				  Tabela PrincipalTotal 
				 */
				 ".$this->getQueryPrincipalTOTAL()."
			)TOTAL, PRODUTO PP, PRODUTO PPP
	        WHERE TOTAL.CDPRODNIVANTE = PP.CDARVPROD
	        AND PP.CDPRODNIVANTE = PPP.CDARVPROD
	        AND ((INSTR(:CDPRODUTOAVO, PPP.CDARVPROD) > 0) OR (:TCDPRODUTOAVO = 'T'))
        ";
        
        
        if($tipoRelatorio == 'A'){
            $query = "SELECT 
                TOTALFINAL.LABELFILIAL,
                TOTALFINAL.GARCON,
                TOTALFINAL.CDFILIAL,
                TOTALFINAL.NMFILIAL,
                TOTALFINAL.CDLOJA,
                TOTALFINAL.NMLOJA,
                TOTALFINAL.CDVENDEDOR AS CDVENDEDOR2,
                TOTALFINAL.NMRAZSOCVEN,
                TOTALFINAL.CDARVPROD,
                TOTALFINAL.NMPRODUTO AS NMPRODUTO2,
                TOTALFINAL.CDPRODNIVANTE,
                TOTALFINAL.CDARVPRODPAI,
                TOTALFINAL.VRUNIT,
                TOTALFINAL.CDARVPRODAVO,
                TOTALFINAL.NMPRODUTOPAI,
                TOTALFINAL.NMPRODUTOAVO,
                SUM(ACR) AS ACR,
                SUM(DES) AS DES,
                SUM(QTPRODVEND) AS QTPRODVEND,
                SUM(VRFINALITEM) AS VRFINALITEM,
                SUM(VRGORJETA) AS VRGORJETA,
                SUM(VRLIQUIDO) AS VRLIQUIDO,
                SUM(VRREPIQUEVENDA) AS VRREPIQUEVENDA,
                SUM(VRREPITEM) AS VRREPITEM,
                SUM(VRTXENTREGA) AS VRTXENTREGA
                FROM (".$query.") TOTALFINAL
            GROUP BY TOTALFINAL.LABELFILIAL,
            TOTALFINAL.GARCON,
            TOTALFINAL.CDFILIAL,
            TOTALFINAL.NMFILIAL,
            TOTALFINAL.CDLOJA,
            TOTALFINAL.NMLOJA,
            TOTALFINAL.CDVENDEDOR,
            TOTALFINAL.NMRAZSOCVEN,
            TOTALFINAL.CDARVPROD,
            TOTALFINAL.VRUNIT,
            TOTALFINAL.NMPRODUTO,
            TOTALFINAL.CDPRODNIVANTE,
            TOTALFINAL.CDARVPRODPAI,
            TOTALFINAL.CDARVPRODAVO,
            TOTALFINAL.NMPRODUTOPAI,
            TOTALFINAL.NMPRODUTOAVO
            ORDER BY TOTALFINAL.LABELFILIAL,
            TOTALFINAL.GARCON, TOTALFINAL.CDARVPROD";
            
        }
        elseif ($tipoRelatorio == 'S') {
            $query = "SELECT TOTALFINAL.LABELFILIAL,
                TOTALFINAL.GARCON,
                TOTALFINAL.CDFILIAL,
                TOTALFINAL.NMFILIAL,
                TOTALFINAL.CDLOJA,
                TOTALFINAL.NMLOJA,
                TOTALFINAL.CDVENDEDOR AS CDVENDEDOR2,
                TOTALFINAL.NMRAZSOCVEN,
                SUM(ACR) AS ACR,
                SUM(DES) AS DES,
                SUM(QTPRODVEND) AS QTPRODVEND,
                SUM(VRFINALITEM) AS VRFINALITEM,
                SUM(VRGORJETA) AS VRGORJETA,
                SUM(VRLIQUIDO) AS VRLIQUIDO,
                SUM(VRREPIQUEVENDA) AS VRREPIQUEVENDA,
                SUM(VRREPITEM) AS VRREPITEM,
                SUM(VRTXENTREGA) AS VRTXENTREGA
                FROM (".$query.") TOTALFINAL
            GROUP BY TOTALFINAL.LABELFILIAL,
            TOTALFINAL.GARCON,
            TOTALFINAL.CDFILIAL,
            TOTALFINAL.NMFILIAL,
            TOTALFINAL.CDLOJA,
            TOTALFINAL.NMLOJA,
            TOTALFINAL.CDVENDEDOR,
            TOTALFINAL.NMRAZSOCVEN
            ORDER BY TOTALFINAL.LABELFILIAL,
            TOTALFINAL.GARCON";
        }
        
        return $query;
        
    }
    
}